'use strict';

const fs = require('fs');
const os = require('os');
const acorn = require('acorn');
const util = require('util');
const acornWalk = require('acorn-walk');

/**
 * @module utils
 */
const writeFile = util.promisify(fs.writeFile);
function isString(value) {
  return Object.prototype.toString.call(value) === '[object String]';
}
function parse(content) {
  let named = false;
  const styles = [];
  const ast = acorn.parse(content, {
    sourceType: 'module',
    ecmaVersion: 'latest'
  });
  acornWalk.simple(ast, {
    ExpressionStatement(node) {
      acornWalk.simple(node, {
        AssignmentExpression({ left }) {
          if (left.type === 'MemberExpression') {
            const { property } = left;
            if (property.type === 'Identifier') {
              if (property.name === 'locals') {
                acornWalk.simple(node, {
                  Property({ key: left, value: right }) {
                    const { value: key } = left;
                    const { value: value } = right;
                    if (isString(key) && isString(value)) {
                      styles.push([key, value]);
                    }
                  }
                });
              }
            }
          }
        }
      });
    },
    ExportNamedDeclaration(node) {
      named = true;
      acornWalk.simple(node, {
        VariableDeclarator({ id, init }) {
          const { name: key } = id;
          const { value } = init;
          if (isString(key) && isString(value)) {
            styles.push([key, value]);
          }
        }
      });
    }
  });
  return [styles, named];
}
function generateTypings(content, banner, eol = os.EOL) {
  const [styles, named] = parse(content);
  const typings = banner ? [banner, ''] : [];
  if (styles.length) {
    if (named) {
      for (const [key, value] of styles) {
        typings.push(`export const ${key}: ${JSON.stringify(value)};`);
      }
    } else {
      typings.push('declare const styles: {');
      for (const [key, value] of styles) {
        typings.push(`  ${key}: ${JSON.stringify(value)}`);
      }
      typings.push('};', '', 'export default styles;');
    }
  }
  return typings.join(eol);
}

exports.generateTypings = generateTypings;
exports.isString = isString;
exports.parse = parse;
exports.writeFile = writeFile;
